/* BioTree.h */

#ifndef __BIOTREE_H
#define __BIOTREE_H

#include "Tree.h"

_CLASSDEF(BioTree)

/*   -         */
class BioTree : public Tree
  {

  public :

    BioTree( RCString  aName="",  double distanceFromParent=1 );
    BioTree( RCBioTree aBioTree );
    BioTree( RCTree    aTree,     double distanceFromParent=1 );
    virtual ~BioTree() {}

    virtual bool isGeneTree    () const { return false; }
    virtual bool isSpeciesTree () const { return false; }

    double distanceFromParent() const { return distFromParent; }
    double distanceFromParent( double dist ) { return distFromParent=dist; }

    void   scaleLengths( double scaleFactor );

    RCBioTree child( uint number ) const
      { return RBioTree(Tree::child(number)); }
    RBioTree child( uint number )
      { return RBioTree(Tree::child(number)); }

    /*    Object'a */
    virtual classType   isA     () const { return bioTreeClass; }
    virtual const char* nameOf  () const { return "BioTree"; }
    virtual void        printOn ( ostream& os ) const;

    /*  "" Clonable */
    virtual Clonable* clone() const { return new BioTree(ME); }

    virtual RBioTree operator = ( RCBioTree aBioTree );
    virtual RTree    operator = ( RCTree    aTree    );

  private :

    double distFromParent;

    virtual int addChild( RCTree childToAdd );
    virtual int addChild( RCString childName="" )
      { return addChild( BioTree( childName ) ); }

//    static RBioTree  treeToBT( RTree  tree );
//    static RCBioTree treeToBT( RCTree tree );

  };

inline BioTree::BioTree( RCString aName, double distFrPr ) :
  Tree(aName),
  distFromParent(distFrPr)
  {}
inline BioTree::BioTree( RCBioTree aBioTree ) :
  Tree(aBioTree.name()),
  distFromParent(aBioTree.distanceFromParent())
  { ME = aBioTree; }
inline BioTree::BioTree( RCTree aTree, double distFrPr ) :
  Tree(aTree.name()),
  distFromParent(distFrPr)
  { ME = aTree; }

//inline RBioTree BioTree::treeToBT( RTree tree )
//  {
//    if ( tree.isA() == treeClass ) // !!! DEBUG !!!
//      cerr << "!   " << tree.nameOf()
//           << "&  BioTree& !" << endl;
//    return RBioTree(tree);
//  }
//inline RCBioTree BioTree::treeToBT( RCTree tree )
//  {
//    if ( tree.isA() == treeClass ) // !!! DEBUG !!!
//      cerr << "!   " << tree.nameOf()
//           << "&  BioTree& !" << endl;
//    return RCBioTree(tree);
//  }

inline int BioTree::addChild( RCTree childToAdd )
  {
    if ( childToAdd.contains(ME) )
      return ERROR;

    return ( childToAdd.isA() == bioTreeClass ? // !!!
             addChildPtr( new BioTree(RCBioTree(childToAdd)) ) :
             addChildPtr( new BioTree(childToAdd) ) );
  }

#endif  // __BIOTREE_H

