/* ClArBase.h */

#ifndef __ClArBase_h
#define __ClArBase_h

#include "Clonable.h"

#define DELTA 16 /*     ClArrayBase::delta */

_CLASSDEF(ClArrayBase)

/*        Clonable */
abstract class ClArrayBase : public Clonable
  {

  public :

    /* //flush */
    ClArrayBase( uns d=DELTA     ) : Clonable(),
      sz(0), fBI(0), el(NULL), delta(d) {}
    ClArrayBase( RCClArrayBase A ) : Clonable(),
      sz(0), fBI(0), el(NULL), delta(A.delta)
      { ME = A; }
    virtual ~ClArrayBase() { flush(); }
    virtual void flush();


    /*   */ int  size()  const { return sz; }  //  (  'count')
    /*   */ int  count() const { return sz; }  //  -

    /*   */ bool isEmpty    () const { return !sz;        }  // 
    /*   */ bool isNotEmpty () const { return !isEmpty(); }  //  
    /*   */ void makeEmpty  ()       { flush();           }  //  

    /*   */ bool indexInRange    ( int i ) const  //  
      { return ( i>=0 && i<sz ); }
    /*   */ bool indexOutOfRange ( int i ) const  //   
      { return !indexInRange(i); }

    /*   */ RClonable  elementAt( int i )
      { return ( indexInRange(i) ? *el[i] : *PClonable(NULL) ); }
    /*   */ RCClonable elementAt( int i ) const
      { return ( indexInRange(i) ? *el[i] : *PClonable(NULL) ); }

    /*   */ bool isEmpty    ( int i ) const
      { return indexInRange(i) && el[i] == NULL; }
    /*   */ bool isNotEmpty ( int i ) const
      { return indexInRange(i) && el[i] != NULL; }


    virtual int  firstIndexOf( RCClonable o, int i=0 ) const;  // indx 

    virtual int  firstBlankIndex   () const   // indx   
      { return fBI; }  //     
    virtual int  firstElementIndex () const;  // indx  
    virtual int  lastElementIndex  () const;  // indx  

    virtual bool hasElement    () const { return fBI>0; }
    virtual bool hasNoElements () const { return !hasElement();          }

    virtual bool contains( RCClonable o ) const  //  ()
      { return ( firstIndexOf(o) < sz ); }


    virtual int  add( RCClonable o )  //  ()
      { return addAt( o, firstBlankIndex() ); }

    virtual void addElements( RCClArrayBase );  //  

    virtual int  remove( RCClonable o )  //  ()
      { return removeAt( firstIndexOf(o) ); }

    virtual int  replace( RCClonable o1, RCClonable o2 )  // 
      { return addAt( o2, firstIndexOf(o1) ); }

    virtual RCClonable firstElement() const  //  
      { return elementAt( firstElementIndex() ); }
    virtual RCClonable lastElement() const   //  
      { return elementAt( lastElementIndex() ); }

    /*   */
    RClonable  operator [] ( int i )       { return elementAt(i); }
    RCClonable operator [] ( int i ) const { return elementAt(i); }
    void       operator =  ( RCClArrayBase );

    /*     Object'a */
    virtual bool isEqual ( RCObject ) const;
    virtual void printOn ( ostream& ) const;

  protected :

    virtual int  addAt    ( RCClonable o,  int i );
    virtual int  removeAt (                int i );
    virtual int  setSize  ( int newSize );

    RPClonable  elPtr ( uint i )       { return el[i]; }
    PCClonable  elPtr ( uint i ) const { return el[i]; }

  private :
    int        sz;
  protected :
    int        fBI;  // first Blank Index
  private :
    PPClonable el;

  public :

    const uint delta;

  };

#undef DELTA

#endif  // __ClArBase_h

