/* ClString.h */

#ifndef __ClString_h
#define __ClString_h

#include <stdlib.h>
#include <iostream>

using namespace std;

#include "Sortable.h"

#define MAX_INPUT_LINE_LENGTH 0x1000 /*     */

_CLASSDEF(String)

/*   String ( ) ============================= */

class String : public Sortable
  {

  public:

    /* / */
    String(       char     chr    );
    String( const char*    ptr="" );
    String( const String&  str    );
    String( bool     x );
    String( int      n ) : len(0), theString(0) { constructor(long(n)); }
    String( unsigned n ) : len(0), theString(0) { constructor((uns long)n); }
    String( long     n ) : len(0), theString(0) { constructor(n); }
    String( uns long n ) : len(0), theString(0) { constructor(n); }
    String( double   x, int ndig=8 );
    virtual ~String() { delete theString; }

    /*   */

    String& operator  = (       char    chr );
    String& operator  = ( const char*   ptr );
    String& operator  = ( const String& str );
    String& operator += (       char    chr );
    String& operator += ( const char*   ptr );
    String& operator += ( const String& str );
    String  operator +  (       char    chr ) const { return String(ME)+=chr; }
    String  operator +  ( const char*   ptr ) const { return String(ME)+=ptr; }
    String  operator +  ( const String& str ) const { return String(ME)+=str; }
    bool    operator == (       char    chr ) const { return isEqual(chr); }
    bool    operator == ( const char*   ptr ) const { return isEqual(ptr); }
    bool    operator == ( const String& str ) const { return isEqual(str); }
    bool    operator != (       char    chr ) const { return !operator==(chr); }
    bool    operator != ( const char*   ptr ) const { return !operator==(ptr); }
    bool    operator != ( const String& str ) const { return !operator==(str); }
    bool    operator <  (       char    chr ) const { return isLessThan(chr); }
    bool    operator <  ( const char*   ptr ) const { return isLessThan(ptr); }
    bool    operator <  ( const String& str ) const { return isLessThan(str); }

    String& operator  = ( int    n )       { return operator =(String(n)); }
    String& operator  = ( uns    n )       { return operator =(String(n)); }
    String& operator  = ( long   n )       { return operator =(String(n)); }
    String& operator  = ( ulong  n )       { return operator =(String(n)); }
    String& operator  = ( double x )       { return operator =(String(x)); }
    String& operator += ( int    n )       { return operator+=(String(n)); }
    String& operator += ( uns    n )       { return operator+=(String(n)); }
    String& operator += ( long   n )       { return operator+=(String(n)); }
    String& operator += ( ulong  n )       { return operator+=(String(n)); }
    String& operator += ( double x )       { return operator+=(String(x)); }
    String  operator +  ( int    n ) const { return String(ME)+=n; }
    String  operator +  ( uns    n ) const { return String(ME)+=n; }
    String  operator +  ( long   n ) const { return String(ME)+=n; }
    String  operator +  ( ulong  n ) const { return String(ME)+=n; }
    String  operator +  ( double x ) const { return String(ME)+=x; }
    String  operator *  ( int    n ) const;
    String  operator *  ( uns    n ) const { return operator*(int(n)); } // ???

    char    charAt      ( int i ) const { return theString[i]; }
    char&   charAt      ( int i )       { return theString[i]; }
    char    operator [] ( int i ) const { return theString[i]; }
    char&   operator [] ( int i )       { return theString[i]; }

    operator const char*  () const { return theString; };
    operator       bool   () const;
    operator       int    () const { return atoi(theString); }
//    operator       uns    () const { return strtoul(theString,0,10); } // ?
    operator       long   () const { return atol(theString); }
    operator       ulong  () const { return strtoul(theString,0,10); }
    operator       double () const { return atof(theString); }

    //  
    bool isEqual    (       char    chr ) const;
    bool isEqual    ( const char*   ptr ) const;
    bool isEqual    ( const String& str ) const;
    bool isEqual    ( const Object& obj ) const;
    bool isLessThan (       char    chr ) const;
    bool isLessThan ( const char*   ptr ) const;
    bool isLessThan ( const String& str ) const;
    bool isLessThan ( const Object& obj ) const;
    bool equalsTo   (       char    chr ) const { return ( isEqual( chr ) ); }
    bool equalsTo   ( const char*   ptr ) const { return ( isEqual( ptr ) ); }
    bool equalsTo   ( const String& str ) const { return ( isEqual( str ) ); }
    bool equalsTo   ( const Object& obj ) const { return ( isEqual( obj ) ); }

    /*      */

    int  length() const { return len; };  //   ( '\0')

    bool isEmpty    () const { return ( len == 0 ); }
    bool isNotEmpty () const { return (!isEmpty()); }
    bool isBlank    () const;
    bool isNotBlank () const { return (!isBlank()); }

    bool startsWith ( RCString substr, int index=0 ) const;
    bool endsWith   ( RCString substr ) const
      { return startsWith( substr, len-substr.len ); }

    int  indexOf     ( char     charToFind,   int fromIndex=0 ) const;
    int  indexOf     ( RCString stringToFind, int formIndex=0 ) const;
    int  lastIndexOf ( char     charToFind,   int fromIndex   ) const;
    int  lastIndexOf ( RCString stringToFind, int fromINdex   ) const;
    int  lastIndexOf ( char     charToFind   ) const
      { return lastIndexOf( charToFind, len-1 ); }
    int  lastIndexOf ( RCString stringToFind ) const
      { return lastIndexOf( stringToFind, len-1 ); }

    bool contains( register const char c ) const
      { return ( indexOf(c) < int(len) ); }
    bool contains( RCString substr ) const
      { return ( indexOf(substr) < int(len) ); }

    /*      */

    String substring( int first_included, int first_not_included ) const;
    String substring( int first ) const { return substring( first, len ); }
    String trim() const;  // without leading and trailing white spaces

    String toLowerCase() const;
    String toUpperCase() const;

    /*    */
    int replace( char oneCharacter, char withAnotherOne );

    //    Object'
    virtual classType   isA    () const { return stringClass; }
    virtual const char* nameOf () const { return "String"; }
    virtual void printOn( ostream& os ) const { os << theString; }

    /*  "" Clonable */
    virtual Clonable* clone() const { return new String(ME); }

    //  "" Sortable
    virtual bool isLessThan( RCSortable s ) const
      { return ( s.isA()==isA() && isLessThan(RCString(s)) ); }

  private :  // -

    unsigned len;        //   (  )
    char*    theString;  //    

    void constructor(     long n );
    void constructor( uns long n );

    void _allocateString();
    void _changeLength( unsigned newLength );
    void _copyFrom( const char* source );

  };


/* inline-  String ============================================= */

inline String::operator bool () const
  {
    if ( ME == "TRUE" || ME == "True" || ME == "true" ||
         ME == "YES"  || ME == "Yes"  || ME == "yes" )
      return true;
    else if ( ME == "FALSE" || ME == "False" || ME == "false" ||
              ME == "NO"    || ME == "No"    || ME == "no" )
      return false;
    else
      return atoi(theString);
  }

/*   */
inline bool String::isEqual( char c ) const
  { return ( len == 1 && theString[0] == c ); }
inline bool String::isEqual    ( RCObject obj ) const
  { return ( obj.isA()==stringClass && isEqual(RString(obj)) ); }
inline bool String::isLessThan( char c ) const
  { return ( theString[0] < c ); }
inline bool String::isLessThan ( RCObject obj ) const
  { return ( obj.isA()==stringClass && isLessThan(RString(obj)) ); }

/*    */
inline istream& operator>>( istream& is, String& string )
  { char buf[MAX_INPUT_LINE_LENGTH]; is >> buf; string = buf; return is; }

/*    */
inline ostream& operator<<( ostream& os, const String& string )
  { string.printOn(os); return os; }

/*  -inline-  String =============================== */

//String nTimes( RCString string, int n );

#undef MAX_INPUT_LINE_LENGTH

#endif  // __ClString_h

