/* GeneTabl.cpp */

#include "GeneTabl.h"

#include "string.h"
#include "GlSttngs.h"

OK_ERROR GenesTable::readFrom( istream& is )
  {
    Element e;
    while ( e.readFrom(is)!=EOF )
      elements.add(e);
    return ( elements.isNotEmpty() ? OK : ERROR );
  }

static int firstNonBlank( istream& is )
  {
    for ( int c=is.get(); ; c=is.get() )
      if ( c!=' ' && c!='\t' && c!='\n' )
        return c;
  }

String GenesTable::whatSpecies( RCString gene ) const
  {
    for ( int i=0; i<elements.size(); i++ )
      if ( RCElement(elements[i]).genes.contains(gene) )
        return RCElement(elements[i]).species;
    return "";
  }

int  GenesTable::speciesNumber( RCString spec ) const
  {
    for ( int i=0; i<elements.size(); i++ )
      if ( RCElement(elements[i]).species == spec )
        return i;
    return ERROR;
  }

OK_ERROR GenesTable::Element::readFrom( istream& is )
  { //   ,    ;)

    IMPORT_STRING( genes2exclude, "GenesToExclude" );
    
    flush();  //   

    if ( is.eof() )
      return ERROR;

    //   
    int c;
    while ( (c=firstNonBlank(is))==';' );
    if ( is.eof() )
      return ERROR;
    for ( ; c!=' '  && c!='\t' && c!='\n' && c!='='  && c!=EOF; c=is.get() )
      species += char(c);
    if ( is.eof() )
      return ERROR;

    c=firstNonBlank(is);
    if ( c=='=' )
      c=firstNonBlank(is);
    else if ( is.eof() )
      return ERROR;
    if ( c=='{' )
      c=firstNonBlank(is);
    else if ( is.eof() )
      return ERROR;

    bool endOfList=false;
    while ( !endOfList )
      {
        String gene;
        bool endOfGene=false;
        while ( !endOfGene && !endOfList )
          {
            switch ( c )
              {
                case EOF: return ERROR;
                case ' ': case '\t': case '\n':
                case ',': case '|': endOfGene = true; break;
                case '}': case ';': endOfList = true; break;
                default : gene+=char(c); c=is.get();  break;
              }
          }
        if ( gene!="" )
          if ( !strstr( genes2exclude, String('"')+gene+'"' ) )
            genes.add(gene);
        if ( !endOfList )
          c=firstNonBlank(is);
      }

    return OK;

  }

void GenesTable::Element::printOn( ostream& os ) const
  {
    os << species << " = {";
    for ( int i=0; i<genes.size(); i++ )
      os << genes[i] << ((i<genes.size()-1)?",":"");
    os << "};";
  }

