/* GeneTree.h */

#ifndef __GENETREE_H
#define __GENETREE_H

#include "BioTree.h"
#include "SrtArray.h"

_CLASSDEF(GeneTree)

/*   */
class GeneTree : public BioTree
  {

  public :

    GeneTree( RCString   aName="",  double _distanceFromParent=1 ) :
      BioTree( aName, _distanceFromParent ), spec("") {}
    GeneTree( RCGeneTree gTree ) :
      BioTree( gTree.name(), gTree.distanceFromParent() ), spec(gTree.spec)
      { ME = gTree; }
    GeneTree( RCTree     aTree,     double _distanceFromParent=1 ) :
      BioTree( aTree.name(), _distanceFromParent ), spec("")
      { ME = aTree; }
    virtual ~GeneTree() {}

    virtual bool isGeneTree() const { return true; }

    RCString species    ()                const { return spec; }
    void     setSpecies  ( RCString sp   )       { spec=sp; }
    String   whatSpecies ( RCString gene ) const;

    SrtArray   geneList() const;
    SrtArray   speciesList() const;
    RGeneTree  gene( RCString geneName )
      { return RGeneTree(RCGeneTree(ME).gene(geneName)); }
    RCGeneTree gene   ( RCString geneName ) const;
    RCGeneTree species( RCString specName ) const;
    void       cutGene ( RCString geneName );
    void       mergeOneSpeciesSubrees();

    RGeneTree child( uint number )
      { return RGeneTree(BioTree::child(number)); }
    RCGeneTree child( uint number ) const
      { return RGeneTree(BioTree::child(number)); }

    RTree     operator=( RCTree     aTree );
    RGeneTree operator=( RCGeneTree aGeneTree );

    /*    Object'a */
    virtual classType   isA     () const { return geneTreeClass; }
    virtual const char* nameOf  () const { return "GeneTree"; }
    virtual void        printOn ( ostream& os ) const;

    /*  "" Clonable */
    virtual Clonable* clone() const { return new GeneTree(ME); }

  private :

    String spec;

    virtual int addChild( RCTree childToAdd );

  };

inline int GeneTree::addChild( RCTree childToAdd )
  {
    if ( childToAdd.contains(ME) )
      return ERROR;

    return ( childToAdd.isA() == geneTreeClass ?
             addChildPtr( new GeneTree(RGeneTree(childToAdd)) ) :
             addChildPtr( new GeneTree(childToAdd) ) );
  }

#endif  // __GENETREE_H

