/* IntMat.h */

#ifndef __IntMat_h
#define __IntMat_h

#include "Clonable.h"

typedef int MatrixElement;
typedef int MatrixIndex;

//   
#define elem MatrixElement
#define imat IntMatrix
#define indx MatrixIndex

#define ERRORMESSAGE( MESSAGE ) { cerr << MESSAGE << endl; exit(1); }

_CLASSDEF(IntMatrix)

class IntMatrix : public Clonable
  {
    elem* el;

  public:

    unsigned const M;  //  
    unsigned const N;  //  

    /* / */
    IntMatrix( uns m, uns n  ); // .
    IntMatrix( RCIntMatrix A ); // .
    ~IntMatrix() { delete el; }

    /*    */
  private :
    bool indexesInRange( indx i, indx j ) const
     { return ( i>=0 && j>=0 && uns(i)<M && uns(j)<N ); }
    bool indexesOutOfRange( indx i, indx j ) const
     { return !indexesInRange(i,j); }
    elem& element( indx i, indx j )
      { return indexesInRange(i,j) ? el[i*N+j] : ( *(elem*)NULL ); }
    elem  element( indx i, indx j ) const
      { return indexesInRange(i,j) ? el[i*N+j] : 0; }

    /*   */
  public :
    elem& operator () ( indx i,indx j )       { return element(i,j); }
    elem  operator () ( indx i,indx j ) const { return element(i,j); }
    bool  operator == ( const imat& A ) const;
    bool  operator != ( const imat& A ) const { return !operator==(A); }
    bool  operator == (       elem  a ) const;
    bool  operator != (       elem  a ) const { return !operator==(a); }
    imat  operator +  ( const imat& A ) const;
    imat  operator -  ( const imat& A ) const;
    imat  operator +  (       elem  a ) const;
    imat  operator -  (       elem  a ) const;
    imat  operator *  (       elem  a ) const;
    imat  operator /  (       elem  a ) const;
    imat& operator =  ( const imat& A );
    imat& operator += ( const imat& A );
    imat& operator -= ( const imat& A );
    imat& operator *= (       elem  a );
    imat& operator /= (       elem  a );

    /* . */
    void  initialize();

    imat  transp  () const; // 

    imat  subMatrix( indx i0, indx j0, indx i1, indx j1 ) const;

    bool  isDiagonal      () const;
    bool  isSymmetric     () const;

    ostream& print    ( ostream &os=cout ) const;
    istream& input    ( istream &is=cin  );

    void  swapLines   ( indx m1, indx m2 );
    void  swapColumns ( indx n1, indx n2 );

    //    Object'
    virtual classType   isA    () const { return intMatrixClass; }
    virtual const char* nameOf () const { return "IntMatrix"; }
    virtual void printOn( ostream& os ) const;

    /*  "" Clonable */
    virtual Clonable* clone() const { return new imat(ME); }

  };

inline imat::imat( unsigned m, unsigned n ) : Clonable(), M(m), N(n)
  {
    if ( M*N == 0 )
      ERRORMESSAGE("Attempt to create a matrix with 0 elements! Exiting.");
    el = new elem[M*N];
    if ( el == NULL )
      ERRORMESSAGE( "Not enough memory to allocate a matrix. Exiting." );
  }
inline imat::imat( const imat& A ) : Clonable(), M(A.M), N(A.N)
  {
    el = new elem[M*N];
    if ( el == NULL )
      ERRORMESSAGE( "Not enough memory to allocate a matrix. Exiting." );
    ME = A;
  }

inline imat     operator*( elem a, const imat& A )   { return ( A*a ); }
inline ostream& print ( const imat& A, ostream& os ) { return A.print(os); }

#undef elem
#undef imat
#undef indx

#undef ERRORMESSAGE

#endif // __IntMat_h

