/* Tree.cpp */
/*
()
() **************************************************************************
() * -inline   Tree                                           *
() **************************************************************************
()
*/

#include "Tree.h"  //     Tree

const Tree nullTree;

/*
   --------------------------------------------------------------------------
   |    Tree                                            |
   --------------------------------------------------------------------------
*/

/*
()       : addChild( Tree childToAdd )  ( );
()    :    -  ;
()   :   ;
()  :   .
()
*/
int Tree::addChild( const RCTree childToAdd )
  {
    if ( childToAdd.contains(ME) )  //    
      return ERROR;  //      

    if ( childToAdd.isA() != isA() )
      cerr << ":   (" << childToAdd.nameOf()
           << ")     (" << nameOf() << ")." << endl;

    const int newChildNum = addChildPtr( new Tree(childToAdd) );

    child(newChildNum).updateLevel();  //    

    return newChildNum;  //    
  }

/*
()       : isEqual()  ();
()    :     ""  -;
()   :    -;
()  : true/false (/).
()
*/
bool Tree::isEqual( RCTree aTree ) const
  {
    bool isEqual=false;  //   ;)

    const uint nOfChildren=numberOfChildren();

    if ( aTree.numberOfChildren() == nOfChildren )
      if ( nOfChildren > 0 )
        {  //    

          // isFree[m] <=>  aTree.child(m) ݣ     child(n)
          bool* isFree = new bool[nOfChildren];
          for ( uint m=0; m<nOfChildren; m++ )
            isFree[m] = true;

          for ( uint n=0; n<nOfChildren; n++ )  //   child(n)
            {
              bool isFound=false;  //  child(n)   aTree.child(m)
              for ( uint m=0; m<nOfChildren && !isFound; m++ )  //  
                if ( isFree[m] )  // ݣ  
                  if ( child(n).isEqual( aTree.child(m) ) )  //  
                    isFound = true,  isFree[m] = false;  //   
              if ( !isFound )  //    
                break;
              else  //    child(n)
                if ( n == nOfChildren-1 )  //    child(n)
                  isEqual = true;  //  
            }

          delete isFree;
        }
      else
        if ( aTree.name() == name() )  //    
          isEqual = true;

    return isEqual;
  }

/*
()       : updateLevel()  ( );
()    :        ;
()   : ---;
()  : .
()
*/
uint Tree::updateLevel()
  {
    const uint newLevel = ( hasParent() ? parent().level()+1 : 0 );
    if ( newLevel != _level )
      {
        _level = newLevel;
        for ( uint n=0; n<numberOfChildren(); n++ )
          child(n).updateLevel();
      }
    return newLevel;
  }

/*
()       : numberOfLeaves()  ( );
()    :    ( ) 
()   : ---;
()  :   ( ).
()
*/
uint Tree::numberOfLeaves() const
  {
    uint numberOfLeaves=0;                           //  
    for ( uint n=0; n<numberOfChildren(); n++ )      //   
      numberOfLeaves += child(n).numberOfLeaves();   // 
    return ( numberOfLeaves + !numberOfLeaves );     //  ((?))
  }

/*
()       : numberOfSubtrees()  ( );
()    :    ()  ( )
()   : ---;
()  :   (Σ ).
()
*/
uint Tree::numberOfSubtrees() const
  {
    uint numberOfSubtrees=1;                            //  
    for ( uint i=0; i<numberOfChildren(); i++ )         //   
      numberOfSubtrees += child(i).numberOfSubtrees();  // 
    return numberOfSubtrees;
  }

/*
()       : height()  ();
()    :    (  );
()   : ---;
()  : .
()
*/
uint Tree::height() const
  {
    uint height=0;
    if ( hasChild() )
      {
        uint maxChildHeight=0;
        for ( uint n=0; n<numberOfChildren(); n++ )
          {
            uint childHeight=child(n).height();
            if ( childHeight > maxChildHeight )
              maxChildHeight = childHeight;
          }
        height = maxChildHeight + 1;
      }
    return height;
  }

/*
()       : maximumPower()  ( );
()    :    ()  ;
()   : ---;
()  :  () .
()
*/
uint Tree::maximumPower() const
  {
    uint maxPower=numberOfChildren();
    for ( uint n=0; n<numberOfChildren(); n++ )
      {
        uint t=child(n).maximumPower();
        if ( t > maxPower )
          maxPower = t;
      }
    return maxPower;
  }

/*
()       : sizeInMemory()  (  );
()    :      ();
()   : ---;
()  :     ().
()
*/
uint Tree::sizeInMemory() const
  {
    uint sizeInMemory = sizeof(Tree) + name().length()+1
                      + numberOfChildren()*sizeof(PTree);
    for ( uint n=0; n<numberOfChildren(); n++ )
      sizeInMemory += child(n).sizeInMemory();
    return sizeInMemory;
  }

/*
()       : root()  ();
()    :    ;
()   : ---;
()  :     .
()
*/
RCTree Tree::root() const
  {
    reg PCTree tp=this;
    for ( reg int i=level(); i>0; i-- )
      tp = PTree(&tp->parent());
    return *tp;
  }

/*
()       : contains( Tree anotherTree )  ();
()    : ,     ;
()   : ;
()  : true / false ( /  ).
()
*/
bool Tree::contains( RCTree anotherTree ) const
  {
    reg PCTree tp=&anotherTree;   //     
    for ( reg int i=anotherTree.level()-level(); i>0; i-- )
      tp = PTree(&tp->parent());  //      
    if ( tp == this )             //  
      return true;
    return false;
  }

/*
()       : printOn( ostream outputStream )  (  ...);
()    :         ;
()   :   ;
()  : ---.
()
*/
void Tree::printOn( ostream& os ) const
  {
    if ( hasChild() )
      {
        os << '(';
        for ( uint i=0; i<numberOfChildren(); i++ )
          os << child(i) << ( (i<numberOfChildren()-1) ? "," : "" );
        os << ')';
      }
    os << name();
  }

/*
()       : childNumber( String name )  ( );
()    : ---;
()   :    ;
()  :       / UNDEFINED.
()
*/
int Tree::childNumber( RCString name ) const
  {
    /*     */
    for ( uint n=0; n<numberOfChildren(); n++ )
      if ( child(n).name() == name )
        return n;
    return UNDEFINED;
  }

/*
   --------------------------------------------------------------------------
   |                                                  |
   --------------------------------------------------------------------------
*/
/*
[]       :   (tree = anotherTree);
[]    :      ;
[]   : -;
[]  : .
[]
*/
RTree Tree::operator=( RCTree anotherTree )
  {
    if ( &anotherTree == this )
      return ME;

    flush();  //    

    name( anotherTree.name() );

    for ( uint n=0; n<anotherTree.numberOfChildren(); n++ )
      addChild( anotherTree.child(n) );  //   

    return ME;
  }

