/* TreeMan.cpp */
/*
()
() **************************************************************************
() * -inline   TreeManipulator                                *
() **************************************************************************
()
*/


#include "TreeMan.h"

/*  #define    */
#define TM TreeManipulator

/*        */
template <class T> void my_swap( T& x, T& y ) { T t=x; x=y; y=t; }


/*
()       : addChildren( children )  ( );
()    :      ;
()   :  (CPArray) ;
()  :    .
()
*/
int TM::addChildren( const RClArray children )
  {
    for ( int i=0; i<children.size(); i++ )
      if ( &children[i] )
    curKidN = hostPtr->addChild( RTree(children[i]) );
    return curKidN;
  }

/*
()       : goToRoot()  (  );
()    :       ;
()   : ---;
()  : ---.
()
*/
void TM::goToRoot()
  {
    for ( int level=host().level(); level>0; level-- )
      goToParent();
  }

/*
()       : goToFirstLeaf()  (   );
()    :       ;
()   : ---;
()  :      ().
()
*/
int TM::goToFirstLeaf()
  {
    const int oldLevel=host().level();
    while ( host().hasChild() )
      goToFirstChild();
    return host().level()-oldLevel;
  }

/*
()       : goToNextLeaf()  (   );
()    :      (.. .) ;
()   : ---;
()  : OK -  ̣ / ERROR -  .
()
*/
int TM::goToNextLeaf()
  {
    if ( host().isNotLeaf() )
      return goToChild(), goToFirstLeaf(), OK;

    while ( hasMoreChild() == false )
      if ( host().hasParent() )
        goToParent();
      else
        return ERROR;

    goToNextChild();
    goToFirstLeaf();

    return OK;
  }

/*
()       : goUpNLevels( int n )  (  n );
()    :    n  ;
()   : ---;
()  : OK -  ̣ / ERROR -  .
()
*/
int TM::goUpNLevels( int n )
  {
    while ( n-->1 )
      if ( host().hasParent() )
        hostPtr = &hostPtr->parent();
      else
        return ERROR;
    return goToParent();
  }

/*
()       : goToCA(...)  (   );
()    :      TreeManipulator' ;
()   : -;
()  :     () / ERROR.
() .. :      
()
*/
int TM::goToCA( TM partner )
  {
    const int oldLevel=host().level();          // . 
    const int oldPartnerLevel=partner->level(); // .  
    const int levelDifference=oldPartnerLevel-oldLevel;  //  

    if ( levelDifference < 0 )                 //   
      goUpNLevels( -levelDifference );         //    
    else if ( levelDifference > 0 )            //   
      partner.goUpNLevels( levelDifference );  //   

    while ( &host() != &partner.host() )  //    
      if ( goToParent() == ERROR || partner.goToParent() == ERROR )
        return ERROR;  // treeMan'    

    return ( oldLevel+oldPartnerLevel-2*host().level() );
  }

/*
()       : goToSubtree( String name )  (  );
()    :       ;
()   : ;
()  : OK -  ̣ / ERROR -  .
()
*/
int TM::goToSubtree( RCString name )
  {
    if ( host().name() == name )  //     
      return OK;                  //   

    if ( host().isLeaf() )        //    
      return ERROR;               //   

    for ( uint n=0; n<host().numberOfChildren(); n++ )  //   
      {
        TreeManipulator treeMan(host().child(n));  //  TM  
        if ( treeMan.goToSubtree(name) == OK )  //   ۣ
          return ( ME=treeMan, OK );  //       
      }

    return ERROR;  //       ۣ -   
  }

/*
()       : numberOfLeaves()  ( );
()    :    ( )  ;
()   : ---;
()  :   ( )  .
()
*//*
TM::numberOfLeaves() const
  {
    int numberOfLeaves=0;  //  
    for ( int i=0; i<host().numberOfChildren(); i++ )  //   
      numberOfLeaves += TM( host().child(i) ).numberOfLeaves();  // 
    return ( numberOfLeaves + !numberOfLeaves );  // 
  }
*/
/*
()       : interchangeChildren(...)  ( );
()    :      TreeManipulator';
()   : -;
()  : OK / ERROR ( / ).
()
*/
OK_ERROR TM::interchangeChildren( RCTreeManipulator partner )
  {
    if ( curKidN == UNDEFINED || partner.curKidN == UNDEFINED )
      return ERROR;  //  ...

    my_swap                                        //  
      (
        hostPtr->childPtr(         curKidN ),      //  
        partner->childPtr( partner.curKidN )       //   
      );
    /*   */ child().meetParent( host() );          //    ...
    partner.child().meetParent( partner.host() );  //     
    if ( partner->level() != host().level() )      //   
      {                                            //   
        /*   */ child().updateLevel();
        partner.child().updateLevel();
      }

    return OK;  //      
  }

/*
()       : takeChildFrom(...)  (  ...);
()    : ""     TreeManipulator';
()   : -;
()  : OK / ERROR ( / ).
()
*/
OK_ERROR TM::takeChildFrom( RCTreeManipulator partner )
  {
    if ( partner.curKidN == UNDEFINED )
      return ERROR;  //  ...

    curKidN = host().addChildPtr( partner->childPtr( partner.curKidN ) );
    if ( curKidN == ERROR )
      return ERROR;
    partner->childPtr( partner.curKidN ) = NULL;  //    :[
    partner->removeChild( partner.curKidN );

    return OK;
  }

/*
()       : nameUntitledSubtrees(...)  (   );
()    :  LR- ,   ;
()   : ,       ͣ;
()  : OK.
()
*/
OK_ERROR TM::nameUntitledSubtrees( RCString beginWith, RCString endWith )
  {
    if ( host().name() == "" )
      host().name( beginWith + endWith );

    for ( uint n=0; n<host().numberOfChildren(); n++ )
      {
        String childName=beginWith;
        switch ( n )
          {
          case  0 : childName += 'L';  break;
          case  1 : childName += 'R';  break;
          default : childName += n;    break;
          };

          TreeManipulator tm( host().child(n) );
          tm.nameUntitledSubtrees( childName, endWith );
      }

    return OK;
  }

#undef TM

